import uuid

from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL
from assembly_payments.types import User, BankAccount
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import UserFactory, BankAccountFactory

fake = Faker()


@requests_mock.Mocker()
class TestBankAccount(PyAssemblyPaymentsTestCase):
    def test_can_get_bank_account(self, mocker):
        self._setup_auth(mocker)

        bank_account = BankAccountFactory()
        mocker.get(f"{TEST_BASE_URL}/bank_accounts/{bank_account['id']}", json={"bank_accounts": bank_account})

        client = AssemblyClient()
        assert client.bank_accounts.get(bank_account['id']) == BankAccount(**bank_account)

    def test_can_get_bank_account_user(self, mocker):
        self._setup_auth(mocker)

        bank_account = BankAccountFactory()
        user = UserFactory()
        mocker.get(f"{TEST_BASE_URL}/bank_accounts/{bank_account['id']}/users", json={"users": user})

        client = AssemblyClient()
        assert client.bank_accounts.get_user(bank_account['id']) == User(**user)

    def test_can_create_bank_account(self, mocker):
        self._setup_auth(mocker)
        data = dict(
            user_id=fake.word(),
            bank_name=fake.word(),
            country="AUS",
            account_name=fake.word(),
            routing_number=fake.word(),
            account_number=fake.word(),
            holder_type="personal",
            account_type=fake.word(),
        )
        bank_account = BankAccount(**BankAccountFactory())

        mocker.post(f"{TEST_BASE_URL}/bank_accounts", json={"bank_accounts": bank_account.model_dump()})

        client = AssemblyClient()
        assert client.bank_accounts.create(**data) == bank_account
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BASE_URL}/bank_accounts"
        assert mocker.last_request.method == "POST"

    def test_can_delete_webhook(self, mocker):
        self._setup_auth(mocker)

        bank_account = BankAccountFactory()
        mocker.delete(f"{TEST_BASE_URL}/bank_accounts/{bank_account['id']}", json={})

        client = AssemblyClient()
        client.bank_accounts.delete(bank_account['id'])
        assert mocker.last_request.url == f"{TEST_BASE_URL}/bank_accounts/{bank_account['id']}"
        assert mocker.last_request.method == "DELETE"
