from datetime import datetime

import pytest
from requests import Response

from assembly_payments.exceptions import PyAssemblyPaymentsInsufficientFundsException, \
    PyAssemblyPaymentsUnprocessableEntityException, PyAssemblyPaymentsInternalErrorException
from assembly_payments.services.base import BaseService


class TestBaseService:
    def test_append_query_params(self):
        now = datetime.now()
        params = {
            "a": "a",
            "a b": "a b",
            "dt": now,
            "d t": now,
        }
        assert BaseService().append_query_params("", params) == f"?a=a&a+b=a+b&dt={now.isoformat()}&d+t={now.isoformat()}"

    def test_422_insufficient_funds(self):
        response = Response()
        response.status_code = 422
        response._content = b"The account you're transacting from does " \
                            b"not have enough funds available"
        with pytest.raises(PyAssemblyPaymentsInsufficientFundsException):
            BaseService()._handle_exceptions(response)

    def test_422_unprocessable_entity(self):
        response = Response()
        response.status_code = 422
        response._content = b""
        with pytest.raises(PyAssemblyPaymentsUnprocessableEntityException):
            BaseService()._handle_exceptions(response)

    def test_504_error(self):
        response = Response()
        response.status_code = 504
        response._content = b"Error"
        with pytest.raises(PyAssemblyPaymentsInternalErrorException) as err:
            BaseService()._handle_exceptions(response)
        assert err.value.args[0] == "Error"
