from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL
from assembly_payments.types import BatchTransaction
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import BatchTransactionFactory

fake = Faker()


@requests_mock.Mocker()
class TestBatchTransactions(PyAssemblyPaymentsTestCase):
    def test_can_list_batch_transactions(self, mocker):
        self._setup_auth(mocker)
        mocker.get(
            f"{TEST_BASE_URL}/batch_transactions",
            json={"batch_transactions": [BatchTransactionFactory()]},
        )

        client = AssemblyClient()
        assert client.batch_transactions.list() != []

    def test_can_get_batch_transaction(self, mocker):
        self._setup_auth(mocker)

        batch_transaction = BatchTransactionFactory()
        mocker.get(
            f"{TEST_BASE_URL}/batch_transactions/{batch_transaction['id']}",
            json=batch_transaction,
        )

        client = AssemblyClient()
        assert client.batch_transactions.get(
            batch_transaction["id"]
        ) == BatchTransaction(**batch_transaction)
