from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL
from assembly_payments.types import User, BPayAccount
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import UserFactory, BPayAccountFactory

fake = Faker()


@requests_mock.Mocker()
class TestBPayAccount(PyAssemblyPaymentsTestCase):
    def test_can_get_bpay_account(self, mocker):
        self._setup_auth(mocker)

        bpay_account = BPayAccountFactory()
        mocker.get(
            f"{TEST_BASE_URL}/bpay_accounts/{bpay_account['id']}",
            json={"bpay_accounts": bpay_account},
        )

        client = AssemblyClient()
        assert client.bpay_accounts.get(bpay_account["id"]) == BPayAccount(
            **bpay_account
        )

    def test_can_get_bpay_account_user(self, mocker):
        self._setup_auth(mocker)

        bpay_account = BPayAccountFactory()
        user = UserFactory()
        mocker.get(
            f"{TEST_BASE_URL}/bpay_accounts/{bpay_account['id']}/users",
            json={"users": user},
        )

        client = AssemblyClient()
        assert client.bpay_accounts.get_user(bpay_account["id"]) == User(**user)

    def test_can_create_bpay_account(self, mocker):
        self._setup_auth(mocker)
        data = dict(
            user_id=fake.word(),
            account_name=fake.word(),
            biller_code=fake.pyint(),
            bpay_crn=str(fake.pyint()),
        )
        bpay_account = BPayAccount(**BPayAccountFactory())

        mocker.post(
            f"{TEST_BASE_URL}/bpay_accounts",
            json={"bpay_accounts": bpay_account.model_dump()},
        )

        client = AssemblyClient()
        assert client.bpay_accounts.create(**data) == bpay_account
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BASE_URL}/bpay_accounts"
        assert mocker.last_request.method == "POST"

    def test_can_delete_bpay_account(self, mocker):
        self._setup_auth(mocker)

        bpay_account = BPayAccountFactory()
        mocker.delete(f"{TEST_BASE_URL}/bpay_accounts/{bpay_account['id']}", json={})

        client = AssemblyClient()
        client.bpay_accounts.delete(bpay_account["id"])
        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/bpay_accounts/{bpay_account['id']}"
        )
        assert mocker.last_request.method == "DELETE"
