from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL
from assembly_payments.types import Company
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import CompanyFactory

fake = Faker()


@requests_mock.Mocker()
class TestCompany(PyAssemblyPaymentsTestCase):
    def test_can_list_companies(self, mocker):
        self._setup_auth(mocker)
        mocker.get(f"{TEST_BASE_URL}/companies", json={"companies": [CompanyFactory()]})

        client = AssemblyClient()
        assert client.companies.list() != []

    def test_can_get_company(self, mocker):
        self._setup_auth(mocker)

        company = CompanyFactory()
        mocker.get(
            f"{TEST_BASE_URL}/companies/{company['id']}", json={"companies": company}
        )

        client = AssemblyClient()
        assert client.companies.get(company["id"]) == Company(**company)

    def test_can_create_company(self, mocker):
        self._setup_auth(mocker)
        data = dict(
            name=fake.word(),
            legal_name=fake.word(),
            tax_number=fake.word(),
            country=fake.email(),
            user_id=fake.word(),
        )

        response_data = CompanyFactory(**data)
        mocker.post(f"{TEST_BASE_URL}/companies", json={"companies": response_data})

        client = AssemblyClient()
        assert client.companies.create(**data).id == response_data["id"]
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BASE_URL}/companies"
        assert mocker.last_request.method == "POST"

    def test_can_update_company(self, mocker):
        self._setup_auth(mocker)
        data = dict(
            name=fake.word(),
            legal_name=fake.word(),
            tax_number=fake.word(),
            country=fake.email(),
            user_id=fake.word(),
        )

        response_data = CompanyFactory(**data)
        company_id = response_data["id"]
        mocker.patch(
            f"{TEST_BASE_URL}/companies/{company_id}", json={"companies": response_data}
        )

        client = AssemblyClient()
        assert client.companies.update(company_id, **data).id == response_data["id"]
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BASE_URL}/companies/{company_id}"
        assert mocker.last_request.method == "PATCH"
