from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL
from assembly_payments.types import DirectDebitAuthority
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import DirectDebitAuthorityFactory, BankAccountFactory

fake = Faker()


@requests_mock.Mocker()
class TestDirectDebitAuthority(PyAssemblyPaymentsTestCase):
    def test_can_list_direct_debit_authority(self, mocker):
        self._setup_auth(mocker)

        bank_account = BankAccountFactory()
        mocker.get(
            f"{TEST_BASE_URL}/direct_debit_authorities?account_id={bank_account['id']}",
            json={"direct_debit_authorities": [DirectDebitAuthorityFactory()]},
        )

        client = AssemblyClient()
        assert client.direct_debit_authority.list(account_id=bank_account["id"]) != []

    def test_can_list_direct_debit_authority_with_params(self, mocker):
        self._setup_auth(mocker)

        bank_account = BankAccountFactory()
        limit = fake.pyint(max_value=200)
        offset = fake.pyint()
        mocker.get(
            f"{TEST_BASE_URL}/direct_debit_authorities?offset={offset}&limit={limit}&account_id={bank_account['id']}",
            json={"direct_debit_authorities": [DirectDebitAuthorityFactory()]},
        )

        client = AssemblyClient()
        assert client.direct_debit_authority.list(account_id=bank_account["id"], limit=limit, offset=offset) != []

    def test_can_get_direct_debit_authority(self, mocker):
        self._setup_auth(mocker)

        direct_debit_authority = DirectDebitAuthorityFactory()
        mocker.get(
            f"{TEST_BASE_URL}/direct_debit_authorities/{direct_debit_authority['id']}",
            json={"direct_debit_authorities": direct_debit_authority},
        )

        client = AssemblyClient()
        assert client.direct_debit_authority.get(
            direct_debit_authority["id"]
        ) == DirectDebitAuthority(**direct_debit_authority)

    def test_can_create_direct_debit_authority(self, mocker):
        self._setup_auth(mocker)
        data = dict(amount=fake.pyint(), account_id=fake.word(),)
        response = DirectDebitAuthorityFactory(amount=data["amount"])

        mocker.post(
            f"{TEST_BASE_URL}/direct_debit_authorities",
            json={"direct_debit_authorities": response},
        )

        client = AssemblyClient()
        assert client.direct_debit_authority.create(**data).amount == data["amount"]
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BASE_URL}/direct_debit_authorities"
        assert mocker.last_request.method == "POST"
