
from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL
from assembly_payments.types import Disbursement, WalletAccount, BankAccount, User, Transaction, Item
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import DisbursementFactory, WalletAccountFactory, BankAccountFactory, UserFactory, \
    TransactionFactory, ItemFactory

fake = Faker()


@requests_mock.Mocker()
class TestDisbursement(PyAssemblyPaymentsTestCase):
    def test_can_list_disbursements(self, mocker):
        self._setup_auth(mocker)
        mocker.get(f"{TEST_BASE_URL}/disbursements", json={"disbursements": [DisbursementFactory()]})

        client = AssemblyClient()
        assert client.disbursements.list() != []

    def test_can_get_disbursement(self, mocker):
        self._setup_auth(mocker)

        disbursement = DisbursementFactory()
        mocker.get(f"{TEST_BASE_URL}/disbursements/{disbursement['id']}", json={"disbursements": disbursement})

        client = AssemblyClient()
        assert client.disbursements.get(disbursement['id']) == Disbursement(**disbursement)

    def test_can_get_disbursement_transactions(self, mocker):
        self._setup_auth(mocker)

        disbursement = DisbursementFactory()
        transaction = TransactionFactory()
        mocker.get(f"{TEST_BASE_URL}/disbursements/{disbursement['id']}/transactions", json={"transactions": [transaction]})

        client = AssemblyClient()
        assert client.disbursements.get_transactions(disbursement['id']) == [Transaction(**transaction)]

    def test_can_get_disbursement_wallet_account(self, mocker):
        self._setup_auth(mocker)

        disbursement = DisbursementFactory()
        wallet_account = WalletAccountFactory()
        mocker.get(f"{TEST_BASE_URL}/disbursements/{disbursement['id']}/wallet_accounts", json={"wallet_accounts": wallet_account})

        client = AssemblyClient()
        assert client.disbursements.get_wallet_account(disbursement['id']) == WalletAccount(**wallet_account)

    def test_can_get_disbursement_bank_account(self, mocker):
        self._setup_auth(mocker)

        disbursement = DisbursementFactory()
        bank_account = BankAccountFactory()
        mocker.get(f"{TEST_BASE_URL}/disbursements/{disbursement['id']}/bank_accounts", json={"bank_accounts": bank_account})

        client = AssemblyClient()
        assert client.disbursements.get_bank_account(disbursement['id']) == BankAccount(**bank_account)

    def test_can_get_disbursement_user(self, mocker):
        self._setup_auth(mocker)

        disbursement = DisbursementFactory()
        user = UserFactory()
        mocker.get(f"{TEST_BASE_URL}/disbursements/{disbursement['id']}/users", json={"users": user})

        client = AssemblyClient()
        assert client.disbursements.get_user(disbursement['id']) == User(**user)

    def test_can_get_disbursement_items(self, mocker):
        self._setup_auth(mocker)

        disbursement = DisbursementFactory()
        item = ItemFactory()
        mocker.get(f"{TEST_BASE_URL}/disbursements/{disbursement['id']}/items",
                   json={"items": [item]})

        client = AssemblyClient()
        assert client.disbursements.get_items(disbursement['id']) == [Item(**item)]