import uuid

from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL
from tests.utils.base import PyAssemblyPaymentsTestCase

fake = Faker()


@requests_mock.Mocker()
class TestTesting(PyAssemblyPaymentsTestCase):
    def test_can_process_npp_payment(self, mocker):
        self._setup_auth(mocker)
        data = dict(
            crn="Customer reference number - reference from wallet",
            payid="PayID - pay_id from wallet",
            amount=10000,
            payee_name="Name of the payee",
            trn="CTBAAUSNXXXN20200211010124845368000",
            clearing_system_transaction_id="CTBAAUSNXXXN20200211010124845368000",
            debtor_name="Test Debtor User",
            debtor_legal_name="Test Debtor Legal Name",
            debtor_bsb="062212",
            debtor_account="06221293921393",
            remittance_information="Customer reference number - reference from wallet",
            pay_id_type="/EMAL",
            end_to_end_id="NOTPROVIDED",
            npp_payin_internal_id="4ba75200-4c9b-11ea-aa0f-6551300d670d",
            pay_id="PayID - pay_id from wallet",
        )

        mocker.patch(f"{TEST_BASE_URL}/testing/wallet_accounts/process_npp_payin")

        client = AssemblyClient()
        client.testing.process_npp_payment(**data)

        assert mocker.last_request.json() == data
        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/testing/wallet_accounts/process_npp_payin"
        )
        assert mocker.last_request.method == "PATCH"

    def test_can_move_pending_batch_transactions_to_batched(self, mocker):
        self._setup_auth(mocker)

        mocker.get(f"{TEST_BASE_URL}/batch_transactions/export_transactions")

        client = AssemblyClient()
        client.testing.move_pending_batch_transactions_to_batched()

        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/batch_transactions/export_transactions"
        )
        assert mocker.last_request.method == "GET"

    def test_can_update_batched_transaction_state(self, mocker):
        self._setup_auth(mocker)
        id = str(uuid.uuid4())
        state = 12700
        data = dict(exported_ids=[id], state=state)

        mocker.patch(f"{TEST_BASE_URL}/batches/{id}/transaction_states")

        client = AssemblyClient()
        client.testing.update_batched_transaction_state(id, state)

        assert mocker.last_request.json() == data
        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/batches/{id}/transaction_states"
        )
        assert mocker.last_request.method == "PATCH"

    def test_can_verify_user(self, mocker):
        self._setup_auth(mocker)
        user_id = str(uuid.uuid4())

        mocker.patch(f"{TEST_BASE_URL}/users/{user_id}/identity_verified")

        client = AssemblyClient()
        client.testing.verify_user(user_id)

        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/users/{user_id}/identity_verified"
        )
        assert mocker.last_request.method == "PATCH"
