
from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient
from tests.utils.base import PyAssemblyPaymentsTestCase

fake = Faker()

@requests_mock.Mocker()
class TestToken(PyAssemblyPaymentsTestCase):
    def test_can_create_new_token(self, mocker):
        client_id = fake.word()
        client_secret = fake.word()
        token = self._setup_auth(mocker)

        client = AssemblyClient(client_id=client_id, client_secret=client_secret, scope="scope")
        assert client.get_auth() == token
        assert mocker.last_request.json() == dict(
            grant_type="client_credentials",
            client_id=client_id,
            client_secret=client_secret,
            scope="scope"
        )

    def test_reuses_token_if_not_expired(self, mocker):
        client_id = fake.word()
        client_secret = fake.word()
        token = self._setup_auth(mocker)

        client = AssemblyClient(client_id=client_id, client_secret=client_secret, scope="scope")
        assert client.get_auth() == token
        assert client.get_auth() == token
        assert len(mocker.request_history) == 1

    def test_refreshes_token_if_expired(self, mocker):
        client_id = fake.word()
        client_secret = fake.word()
        token = self._setup_auth(mocker, expires_in=0)

        client = AssemblyClient(client_id=client_id, client_secret=client_secret, scope="scope")
        assert client.get_auth() == token
        assert client.get_auth() == token
        assert len(mocker.request_history) == 2
