from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BETA_URL
from assembly_payments.types import Transaction, TransactionSupplementaryData
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import TransactionFactory, TransactionSupplementaryDataFactory

fake = Faker()


@requests_mock.Mocker()
class TestTransactions(PyAssemblyPaymentsTestCase):
    def test_can_list_transactions(self, mocker):
        self._setup_auth(mocker)
        mocker.get(
            f"{TEST_BETA_URL}/transactions",
            json={"transactions": [TransactionFactory()]},
        )

        client = AssemblyClient()
        assert client.transactions.list() != []

    def test_can_get_transaction(self, mocker):
        self._setup_auth(mocker)

        transaction = TransactionFactory()
        mocker.get(
            f"{TEST_BETA_URL}/transactions/{transaction['id']}",
            json=dict(transactions=transaction),
        )

        client = AssemblyClient()
        assert client.transactions.get(transaction["id"]) == Transaction(**transaction)

    def test_can_get_transaction_supplementary_data(self, mocker):
        self._setup_auth(mocker)

        transaction_supplementary_data = TransactionSupplementaryDataFactory()
        mocker.get(
            f"{TEST_BETA_URL}/transactions/{transaction_supplementary_data['id']}/supplementary_data",
            json=transaction_supplementary_data,
        )

        client = AssemblyClient()
        assert client.transactions.get_transaction_supplementary_data(transaction_supplementary_data["id"]) == TransactionSupplementaryData(**transaction_supplementary_data)
