
from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL, TEST_BETA_URL
from assembly_payments.types import VirtualAccount
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import WalletAccountFactory, VirtualAccountFactory

fake = Faker()


@requests_mock.Mocker()
class TestVirtualAccount(PyAssemblyPaymentsTestCase):
    def test_can_get_virtual_account(self, mocker):
        self._setup_auth(mocker)

        virtual_account = VirtualAccountFactory()
        mocker.get(f"{TEST_BETA_URL}/virtual_accounts/{virtual_account['id']}", json=virtual_account)

        client = AssemblyClient()
        assert client.virtual_accounts.get(virtual_account['id']) == VirtualAccount(**virtual_account)

    def test_can_create_virtual_account(self, mocker):
        self._setup_auth(mocker)

        wallet_account = WalletAccountFactory()
        virtual_account = VirtualAccountFactory()
        mocker.post(f"{TEST_BETA_URL}/wallet_accounts/{wallet_account['id']}/virtual_accounts", json=virtual_account)

        client = AssemblyClient()
        assert client.virtual_accounts.create(wallet_account_id=wallet_account['id']).id == virtual_account['id']
        assert mocker.last_request.url == f"{TEST_BETA_URL}/wallet_accounts/{wallet_account['id']}/virtual_accounts"
        assert mocker.last_request.method == "POST"